@@ -1,4 +1,3 @@
-
 # encoding: utf-8
 #
 #
@@ -34,18 +33,20 @@ DEBUG = False
 # MAX_DATETIME = datetime(2286, 11, 20, 17, 46, 39)
 DEFAULT_WAIT_TIME = 10 * 60  # SECONDS
 
-datetime.strptime('2012-01-01', '%Y-%m-%d')  # http://bugs.python.org/issue7980
+datetime.strptime("2012-01-01", "%Y-%m-%d")  # http://bugs.python.org/issue7980
 
 
 class Queue(object):
     """
-     SIMPLE MULTI-THREADED QUEUE
+    SIMPLE MULTI-THREADED QUEUE
 
-     (multiprocessing.Queue REQUIRES SERIALIZATION, WHICH
-     IS DIFFICULT TO USE JUST BETWEEN THREADS)
+    (multiprocessing.Queue REQUIRES SERIALIZATION, WHICH
+    IS DIFFICULT TO USE JUST BETWEEN THREADS)
     """
 
-    def __init__(self, name, max=None, silent=False, unique=False, allow_add_after_close=False):
+    def __init__(
+        self, name, max=None, silent=False, unique=False, allow_add_after_close=False
+    ):
         """
         max - LIMIT THE NUMBER IN THE QUEUE, IF TOO MANY add() AND extend() WILL BLOCK
         silent - COMPLAIN IF THE READERS ARE TOO SLOW
@@ -54,9 +55,11 @@ class Queue(object):
         self.name = name
         self.max = coalesce(max, 2 ** 10)
         self.silent = silent
-        self.allow_add_after_close=allow_add_after_close
+        self.allow_add_after_close = allow_add_after_close
         self.unique = unique
-        self.closed = Signal("stop adding signal for " + name)  # INDICATE THE PRODUCER IS DONE GENERATING ITEMS TO QUEUE
+        self.closed = Signal(
+            "stop adding signal for " + name
+        )  # INDICATE THE PRODUCER IS DONE GENERATING ITEMS TO QUEUE
         self.lock = Lock("lock for queue " + name)
         self.queue = deque()
 
@@ -161,12 +164,17 @@ class Queue(object):
 
         :param timeout:  IN SECONDS
         """
+        if len(self.queue) < self.max:
+            return
+
         wait_time = 5
 
-        (DEBUG and len(self.queue) > 1 * 1000 * 1000) and Log.warning("Queue {{name}} has over a million items")
+        (
+            DEBUG and len(self.queue) > 1 * 1000 * 1000
+        ) and Log.warning("Queue {{name}} has over a million items")
 
         start = time()
-        stop_waiting = Till(till=start+coalesce(timeout, DEFAULT_WAIT_TIME))
+        stop_waiting = Till(till=start + coalesce(timeout, DEFAULT_WAIT_TIME))
 
         while not self.closed and len(self.queue) >= self.max:
             if stop_waiting:
@@ -179,10 +187,11 @@ class Queue(object):
                 if not stop_waiting and len(self.queue) >= self.max:
                     now = time()
                     Log.alert(
-                        "Queue with name {{name|quote}} is full with ({{num}} items), thread(s) have been waiting {{wait_time}} sec",
+                        "Queue with name {{name|quote}} is full with ({{num}} items),"
+                        " thread(s) have been waiting {{wait_time}} sec",
                         name=self.name,
                         num=len(self.queue),
-                        wait_time=now-start
+                        wait_time=now - start,
                     )
 
     def __len__(self):
@@ -238,7 +247,7 @@ class Queue(object):
             elif not self.queue:
                 return None
             else:
-                v =self.queue.popleft()
+                v = self.queue.popleft()
                 if v is THREAD_STOP:  # SENDING A STOP INTO THE QUEUE IS ALSO AN OPTION
                     self.closed.go()
                 return v
@@ -258,14 +267,36 @@ class Queue(object):
 
 class PriorityQueue(Queue):
     """
-        ADDS ITEMS TO THEIR PRIORITY AND POP'S THE HIGHEST PRIORITY VALUE (UNLESS REQUESTED OTHERWISE)
+    ADDS ITEMS TO THEIR PRIORITY AND POP'S THE HIGHEST PRIORITY VALUE (UNLESS REQUESTED OTHERWISE)
     """
-    def __init__(self, name, numpriorities, max=None, silent=False, unique=False, allow_add_after_close=False):
-        Queue.__init__(self, name=name, max=max, silent=silent, unique=False, allow_add_after_close=False)
+
+    def __init__(
+        self,
+        name,
+        numpriorities,
+        max=None,
+        silent=False,
+        unique=False,
+        allow_add_after_close=False,
+    ):
+        Queue.__init__(
+            self,
+            name=name,
+            max=max,
+            silent=silent,
+            unique=False,
+            allow_add_after_close=False,
+        )
 
         self.numpriorities = numpriorities
         self.queue = [
-            Queue(name=name, max=max, silent=silent, unique=False, allow_add_after_close=False)
+            Queue(
+                name=name,
+                max=max,
+                silent=silent,
+                unique=False,
+                allow_add_after_close=False,
+            )
             for _ in range(numpriorities)
         ]
 
@@ -394,13 +425,12 @@ class PriorityQueue(Queue):
             elif not self.queue:
                 return None
             else:
-                v =self.pop(priority=priority)
+                v = self.pop(priority=priority)
                 if v is THREAD_STOP:  # SENDING A STOP INTO THE QUEUE IS ALSO AN OPTION
                     self.closed.go()
                 return v
 
 
-
 class ThreadedQueue(Queue):
     """
     DISPATCH TO ANOTHER (SLOWER) queue IN BATCHES OF GIVEN size
@@ -412,14 +442,14 @@ class ThreadedQueue(Queue):
         name,
         slow_queue,  # THE SLOWER QUEUE
         batch_size=None,  # THE MAX SIZE OF BATCHES SENT TO THE SLOW QUEUE
-        max_size=None,   # SET THE MAXIMUM SIZE OF THE QUEUE, WRITERS WILL BLOCK IF QUEUE IS OVER THIS LIMIT
+        max_size=None,  # SET THE MAXIMUM SIZE OF THE QUEUE, WRITERS WILL BLOCK IF QUEUE IS OVER THIS LIMIT
         period=None,  # MAX TIME (IN SECONDS) BETWEEN FLUSHES TO SLOWER QUEUE
         silent=False,  # WRITES WILL COMPLAIN IF THEY ARE WAITING TOO LONG
         error_target=None  # CALL error_target(error, buffer) **buffer IS THE LIST OF OBJECTS ATTEMPTED**
-                           # BE CAREFUL!  THE THREAD MAKING THE CALL WILL NOT BE YOUR OWN!
-                           # DEFAULT BEHAVIOUR: THIS WILL KEEP RETRYING WITH WARNINGS
+        # BE CAREFUL!  THE THREAD MAKING THE CALL WILL NOT BE YOUR OWN!
+        # DEFAULT BEHAVIOUR: THIS WILL KEEP RETRYING WITH WARNINGS
     ):
-        if period !=None and not isinstance(period, (int, float, long)):
+        if period != None and not isinstance(period, (int, float, long)):
             Log.error("Expecting a float for the period")
         period = coalesce(period, 1)  # SECONDS
         batch_size = coalesce(batch_size, int(max_size / 2) if max_size else None, 900)
@@ -429,7 +459,18 @@ class ThreadedQueue(Queue):
 
         self.name = name
         self.slow_queue = slow_queue
-        self.thread = Thread.run("threaded queue for " + name, self.worker_bee, batch_size, period, error_target) # parent_thread=self)
+        self.thread = (
+            Thread
+            .run(
+                f"threaded queue for {name}",
+                self.worker_bee,
+                batch_size,
+                period,
+                error_target,
+                parent_thread=self,
+            )
+            .release()
+        )
 
     def worker_bee(self, batch_size, period, error_target, please_stop):
         please_stop.then(lambda: self.add(THREAD_STOP))
@@ -478,14 +519,10 @@ class ThreadedQueue(Queue):
                         Log.warning(
                             "`error_target` should not throw, just deal",
                             name=self.name,
-                            cause=f
+                            cause=f,
                         )
                 else:
-                    Log.warning(
-                        "Unexpected problem",
-                        name=self.name,
-                        cause=cause
-                    )
+                    Log.warning("Unexpected problem", name=self.name, cause=cause)
 
             try:
                 if len(_buffer) >= batch_size or next_push:
@@ -502,14 +539,14 @@ class ThreadedQueue(Queue):
                         Log.warning(
                             "`error_target` should not throw, just deal",
                             name=self.name,
-                            cause=f
+                            cause=f,
                         )
                 else:
                     Log.warning(
                         "Problem with {{name}} pushing {{num}} items to data sink",
                         name=self.name,
                         num=len(_buffer),
-                        cause=cause
+                        cause=cause,
                     )
 
         if _buffer:
@@ -531,7 +568,9 @@ class ThreadedQueue(Queue):
             if not self.closed:
                 self.queue.extend(values)
             if not self.silent:
-                Log.note("{{name}} has {{num}} items", name=self.name, num=len(self.queue))
+                Log.note(
+                    "{{name}} has {{num}} items", name=self.name, num=len(self.queue)
+                )
         return self
 
     def __enter__(self):
@@ -547,6 +586,3 @@ class ThreadedQueue(Queue):
         self.add(THREAD_STOP)
         self.thread.join()
         return self
-
-
-
